/*
[The "BSD licence"]
Copyright (c) 2005 Kunle Odutola
Copyright (c) 2003-2005 Terence Parr
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
3. The name of the author may not be used to endorse or promote products
derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


namespace Antlr.StringTemplate.Language
{
	using System;
	using IList				= System.Collections.IList;
	using IToken			= antlr.IToken;
	using CommonToken		= antlr.CommonToken;
	using TokenCreator		= antlr.TokenCreator;
	using CollectionUtils	= Antlr.StringTemplate.Collections.CollectionUtils;
	
	public class StringTemplateToken : CommonToken
	{
		new public static readonly StringTemplateToken.StringTemplateTokenCreator Creator = new StringTemplateTokenCreator();

		/// <summary>
		/// Track any args for anonymous templates like
		/// <tokens,rules:{t,r | <t> then <r>}>
		/// The lexer in action.g returns a single token ANONYMOUS_TEMPLATE
		/// and so I need to have it parse args in the lexer and make them
		/// available for when I build the anonymous template.
		/// </summary>
		public IList args;
		
		public StringTemplateToken() : base()
		{
		}
		
		public StringTemplateToken(int type, string text) : base(type, text)
		{
		}
		
		public StringTemplateToken(string text) : base(text)
		{
		}
		
		public StringTemplateToken(int type, string text, IList args) : base(type, text)
		{
			this.args = args;
		}
		
		public override string ToString()
		{
			return base.ToString() + "; args=" + CollectionUtils.ListToString(args);
		}

		public class StringTemplateTokenCreator : TokenCreator
		{
			public StringTemplateTokenCreator() {}

			/// <summary>
			/// Returns the fully qualified name of the Token type that this
			/// class creates.
			/// </summary>
			public override string TokenTypeName
			{
				get 
				{ 
					return typeof(StringTemplateToken).FullName;; 
				}
			}

			/// <summary>
			/// Constructs a <see cref="Token"/> instance.
			/// </summary>
			public override IToken Create()
			{
				return new StringTemplateToken();
			}
		}
	}
}